#include <stdlib.h>
#include "c99tree-cc.h"
#include "gtx.h"
#include "tree_print.h"
#include "scope.h"

const char *c99c_verbose_option_names[] = {
	"typedef",
	NULL
};

void c99c_error_fprintf(c99tree_cc_t *ctx, const char *s, FILE *f)
{
	char *fl;

	if (ctx->loc_current->file != NULL)
		fl = ctx->loc_current->file->path;
	else
		fl = "<stdin>";

	fprintf(f, "yyerror: %s", s);
	if (ctx->error_hint != NULL)
		fprintf(f, " (%s)", ctx->error_hint);
	fprintf(f, " at %s", fl);

	if (ctx->loc_current->sline == ctx->loc_current->eline)
		fprintf(f, ":%d %d-%d", ctx->loc_current->sline, ctx->loc_current->scol, ctx->loc_current->ecol-1);
	else
		fprintf(f, ":%d-%d", ctx->loc_current->sline, ctx->loc_current->eline);

	fprintf(f, " (byteoffs: %d-%d)\n", ctx->loc_current->soffs, ctx->loc_current->eoffs);
}

c99c_print_cb_t *c99c_set_output_format(c99tree_cc_t *ctx, const char *fmt)
{
	if (strcmp(fmt, "xml") == 0)
		return ctx->Nprint_cb = C99Nprint_xml;
	if (strcmp(fmt, "plist") == 0)
		return ctx->Nprint_cb = C99Nprint_plist;
	if (strcmp(fmt, "json") == 0)
		return ctx->Nprint_cb = C99Nprint_json;
	if (strcmp(fmt, "perl") == 0)
		return ctx->Nprint_cb = C99Nprint_perl;
	if (strcmp(fmt, "ind") == 0)
		return ctx->Nprint_cb = C99Nprint_ind;
	if (strcmp(fmt, "awk") == 0)
		return ctx->Nprint_cb = C99Nprint_awk;
	if (strcmp(fmt, "web3") == 0)
		return ctx->Nprint_cb = C99Nprint_web3;
	if (strcmp(fmt, "c") == 0)
		return ctx->Nprint_cb = C99Nprint_c;
	if (strcmp(fmt, "dot") == 0)
		return ctx->Nprint_cb = C99Nprint_dot;
	return NULL;
}

int c99c_set_verbose(c99tree_cc_t *ctx, const char *name)
{
	int n;

	if (strcmp(name, "all") == 0) {
		for(n = 0; n < VO_end; n++)
			ctx->verbose_options[n]++;
		return 0;
	}

	for(n = 0; n < VO_end; n++) {
		if (strcmp(name, c99c_verbose_option_names[n]) == 0) {
			ctx->verbose_options[n]++;
			return 0;
		}
	}

	return -1;
}

void c99tree_cc_init(c99tree_cc_t *ctx)
{
	memset(ctx, 0, sizeof(c99tree_cc_t));
	ctx->Nprint_cb = C99Nprint_ind;

	c99c_gtx_init(ctx);

	ctx->loc_format = "%m:%M@%f:%l";

	c99c_tree_init(ctx);
	c99c_scope_enter(ctx); /* master scope for global typedefs */
}

void c99tree_cc_uninit(c99tree_cc_t *ctx)
{
	c99c_loc_destroy(ctx);
	c99c_tree_uninit(ctx);
	c99c_scope_leave(ctx);
}

